/*
MySerial_h - Hardware serial library for Wiring
Copyright (c) 2006 Nicholas Zambetti.  All right reserved.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

Modified 28 September 2010 by Mark Sproul
*/

#ifndef MySerial_h
#define MySerial_h


#define DEC 10
#define HEX 16
#define OCT 8
#define BIN 2
#define BYTE 0


#define RX_BUFFER_SIZE 256		//Max 256, power of 2
#define PTRL_BLOCK_SIZE 32

#define SERIAL_NO_DATA 0xff

#define RTS_CHAR 0x12			//Request to send char
#define CTS_CHAR 0x14			//Clear to send char

extern char rx_buffer[RX_BUFFER_SIZE];
extern unsigned char rx_buffer_head;
extern unsigned char rx_buffer_tail;

class MySerial //: public Stream
{

public:
	MySerial();
	void begin(long);
	void end();
	int peek(void);
	char read(void);
	void flush(void);
	volatile bool overflow;
	volatile bool rts_flag;

	// Send CTS char string

	FORCE_INLINE void send_cts(){
		rts_flag = false;
		write(CTS_CHAR);
	}

	FORCE_INLINE int free_room(void)
	{
		return (int)(RX_BUFFER_SIZE-((RX_BUFFER_SIZE + rx_buffer_head - rx_buffer_tail) % RX_BUFFER_SIZE));
	}

	// Check if available

	FORCE_INLINE int available(void)
	{
		int n = (int)(RX_BUFFER_SIZE + rx_buffer_head - rx_buffer_tail) % RX_BUFFER_SIZE;
                if (n == 0 && rts_flag) send_cts();
                return n;
	}

	// Write a char to serial

	FORCE_INLINE void write(unsigned char c)
	{
		while (!((UCSR0A) & (1 << UDRE0)))
			;

		UDR0 = c;
	}
	

	FORCE_INLINE void checkRx(void)
	{
		if((UCSR0A & (1<<RXC0)) != 0) {
			char data  =  UDR0;
			unsigned char next_head;
			if (data == RTS_CHAR){
				rts_flag = true;
			} else {
				// if we should be storing the received character into the location
				// just before the tail (meaning that the head would advance to the
				// current location of the tail), we're about to overflow the buffer
				// and so we don't write the character or advance the head.
				next_head = (unsigned char)(rx_buffer_head + 1) % RX_BUFFER_SIZE;
				if (next_head != rx_buffer_tail) {
					rx_buffer[rx_buffer_head] = data;
					rx_buffer_head = next_head;
				}else {
					overflow = true;
				}
			}
		}
	}


private:
	void printNumber(unsigned long, uint8_t);
	void printFloat(double, uint8_t);


public:

	FORCE_INLINE void write(const char *str)
	{
		while (*str)
			write((unsigned char)*str++);
	}


	FORCE_INLINE void write(const unsigned char *buffer, size_t size)
	{
		while (size--)
			write((unsigned char)*buffer++);
	}

	FORCE_INLINE void print(const String &s)
	{
		for (int i = 0; i < (int)s.length(); i++) {
			write((unsigned char)s[i]);
		}
	}

	FORCE_INLINE void print(const char *str)
	{
		write(str);
	}
	void print(char, int = BYTE);
	void print(unsigned char, int = BYTE);
	void print(int, int = DEC);
	void print(unsigned int, int = DEC);
	void print(long, int = DEC);
	void print(unsigned long, int = DEC);
	void print(double, int = 2);

	void println(const String &s);
	void println(const char[]);
	void println(char, int = BYTE);
	void println(unsigned char, int = BYTE);
	void println(int, int = DEC);
	void println(unsigned int, int = DEC);
	void println(long, int = DEC);
	void println(unsigned long, int = DEC);
	void println(double, int = 2);
	void println(void);
};

extern MySerial MSerial;

#endif
