/*
laser_control.h - laser control code
Part of TwinTeeh Firmware

Copyright (c) 2015 V.Ayllon.  All rights reserved.

TwinTeeth Firmware is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
to anyone for the consequences of using it or for whether it serves any
particular purpose or works at all, unless he says so in writing.  Refer
to the GNU General Public License, Version 2 or later, for full details.

Everyone is granted permission to copy, modify and redistribute
TwinTeeth Firmware, but only under the conditions described in the GNU General Public
License.  A copy of this license is supposed to have been
given to you along with TwinTeeth Firmware so you can know your rights and
responsibilities.  It should be in a file named COPYING.  Among other
things, the copyright notice and this notice must be preserved on all
copies.

- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/

#ifndef LASER_CONTROL_H
#define LASER_CONTROL_H

#define LASER_OFF	0
#define LASER_BLUE	1
#define LASER_RED	2
#define LASER_IR	3

#define MAX_LASER_RED_PWR	55		// Red laser destroy itself at high power.

void laser_init();
void laser_end();
void move_focus_servo(uint8_t pos);

extern uint8_t current_diode;

/**
** set_laser_on() - Fires the laser modifing power level 0 - 128
** It's assumed that laser was previously turned on with turn_laser_on()
**/
FORCE_INLINE void set_laser_power(uint8_t level){	
	TCCR4A |= (1 << COM4A1);
	OCR4AH = 0;
	OCR4AL = level;
}


/**
** set_laser_power_off() - Stop the laser but modifing power level
** It's assumed that laser was previously turned on with turn_laser_on().
**/
FORCE_INLINE void set_laser_power_off(){

	TCCR4A |= (1 << COM4A1);
	OCR4AH = 0;
	OCR4AL = 0;
}
/**
** get_laser_power() - Get the intensity of laser
**/
FORCE_INLINE uint8_t get_laser_power(){
	return OCR4AL;
}

/**
** turn_laser() - Fires the laser diode selected on/off
** SEL1  SEL2
**  L     L  = OFF
**  H     L  = RED
**  L     H  = IR
**  H     H  = BLUE
**/
FORCE_INLINE void turn_laser(uint8_t mode){
	switch(mode){
	case LASER_RED:
		if (get_laser_power() > MAX_LASER_RED_PWR)
			set_laser_power(MAX_LASER_RED_PWR);
		WRITE(LASER_SEL1_PIN,HIGH);
		WRITE(LASER_SEL2_PIN,LOW);
		break;
	case LASER_IR:
		WRITE(LASER_SEL1_PIN,LOW);
		WRITE(LASER_SEL2_PIN,HIGH);
		break;
	case LASER_BLUE:
		WRITE(LASER_SEL1_PIN,HIGH);
		WRITE(LASER_SEL2_PIN,HIGH);
		break;
	case LASER_OFF:
	default:
		WRITE(LASER_SEL1_PIN,LOW);
		WRITE(LASER_SEL2_PIN,LOW);
		break;

	}
}

/**
 ** turn_laser_on() - Turn on current laser diode
 **/
FORCE_INLINE void turn_laser_on() {
	turn_laser(current_diode);
}
/**
 ** turn_laser_off() - Turn off current diode
 **/
FORCE_INLINE void turn_laser_off() {
	turn_laser(LASER_OFF);
}

/**
 ** set_laser_diode() - Sets the current laser diode
 **/
FORCE_INLINE void set_laser_diode(uint8_t d){
	current_diode = d;
}
/**
 ** get_laser_diode() - Gets the current laser diode
 **/
FORCE_INLINE uint8_t get_laser_diode(){
	return current_diode;
}
/**
** set_focus_pos() - Moves the focus len to the focus position
**/
FORCE_INLINE void set_focus_pos(uint8_t pos){
	TCCR3A |= (1 << COM3A1);
	OCR3AH = 0;
	OCR3AL = pos;
}

#endif